﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGrid;
using System.Globalization;
using System.Diagnostics;

namespace FractionalConverter
{
    #region frmMain class
    public partial class frmMain : Form
    {
        private DataTable table = null;

        public frmMain()
        {
            InitializeComponent();

            this.grid.Dock = DockStyle.Fill;
            this.grid.InitializeLayout += new InitializeLayoutEventHandler(this.grid_InitializeLayout);
            this.grid.DataSource = this.Data;
        }

        private void grid_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            ColumnsCollection columns = e.Layout.Bands[0].Columns;
            foreach ( UltraGridColumn column in columns )
            {
                string key = column.Key;

                switch ( key )
                {
                    //  Assign the data filter
                    case "Double":
                        column.Editor.DataFilter = new FractionalConversionDataFilter( 4 );
                        break;

                }
            }
        }

        private DataTable Data
        {
            get
            {
                if ( this.table == null )
                {
                    this.table = new DataTable();

                    //  Add columns
                    this.table.Columns.Add( "Double", typeof(double) );

                    //  Add rows
                    this.table.Rows.Add( new object[]{ "1.375" } );
                    this.table.Rows.Add( new object[]{ "2.3125" } );
                    this.table.Rows.Add( new object[]{ "1.5" } );
                    this.table.Rows.Add( new object[]{ "6.625" } );

                }

                return this.table;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if ( this.grid.ActiveCell != null &&
                 this.grid.ActiveCell.Value is double )
                this.textBox1.Text = this.grid.ActiveCell.Value.ToString();
        }
    }
    #endregion frmMain class

    #region FractionalConversionDataFilter class
    /// <summary>
    /// IEditorDataFilter implementation which performs bi-directional
    /// conversions for fractional representations of double values.
    /// </summary>
    public class FractionalConversionDataFilter : IEditorDataFilter
    {
        #region Member variables
        
        private int precision = 2;

        #endregion Member variables

        #region Constructors

        /// <summary>
        /// Creates a new instance.
        /// </summary>
        public FractionalConversionDataFilter( int precision )
        {
            this.precision = precision > 0 ? precision : 2;
        }

        #endregion Constructors

        #region IEditorDataFilter

        object IEditorDataFilter.Convert(EditorDataFilterConvertArgs conversionArgs)
        {
            object rawValue = conversionArgs.Value;
            ConversionDirection direction = conversionArgs.Direction;

            switch ( direction )
            {
                //  "Convert" the double value to a string of the form "x y/z"
                //  where x represents the integral component of the number,
                //  y the numerator, and z the denominator
                case ConversionDirection.EditorToOwner:
                {                    
                    conversionArgs.Handled = true;
                    
                    
                    if ( rawValue is string )
                    {
                        string stringVal = rawValue as string;
                        return FractionalConversionDataFilter.ToDouble( stringVal, this.precision );
                    }
                }
                break;

                //  Reverse the conversion on the 'EditorToOwner' direction
                case ConversionDirection.OwnerToEditor:
                {                    
                    conversionArgs.Handled = true;

                    if ( rawValue is double )
                    {
                        double doubleVal = (double)rawValue;
                        return FractionalConversionDataFilter.ToFraction( doubleVal, this.precision );
                    }
                }
                break;
            }

            return conversionArgs.Value;
        }

        #endregion IEditorDataFilter

        private static string ToFraction( double value, int precision )
        {
            //  Break out the integral and fractional components
            int integer = (int)Math.Truncate(value);
            double fraction = value - integer;

            //  Round the fractional component
            fraction = Math.Round( fraction, precision );

            //  Get the GCD of the fractional portion and the closest power of ten
            int y = (int)Math.Pow(10, precision);
            int x = (int)(fraction * y);
            int gcd = GCD(x, y);

            //  Get the string representations of the numerator and denominator
            string numerator = (x / gcd).ToString();
            string denominator = (y / gcd).ToString();

            //  Return the compiled result
            if ( integer == 0 )
                return string.Format( "{0}/{1}", numerator, denominator );
            else
                return string.Format( "{0} {1}/{2}", integer, numerator, denominator );
        }

        private static double ToDouble( string value, int precision )
        {
            System.Text.RegularExpressions.Match match = null;

            //  Get the integral component
            match = System.Text.RegularExpressions.Regex.Match( value, @"\d\s?" );
            string integer = match != null && match.Success ? match.Value.Trim() : "0";

            //  Get the numerator...there is some regex way to remove the slash but I can't remember it
            match = System.Text.RegularExpressions.Regex.Match( value, @"\d*/" );
            string numerator = match != null && match.Success ? match.Value.Trim() : "0";
            numerator = numerator.Replace("/", string.Empty);

            //  Get the denominator
            match = System.Text.RegularExpressions.Regex.Match( value, @"/\d*" );
            string denominator = match != null && match.Success ? match.Value.Trim() : "0";
            denominator = denominator.Replace("/", string.Empty);

            //string[] strings = value.Split(" ".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);

            int i = int.Parse(integer);
            int n = int.Parse(numerator);
            int d = int.Parse(denominator);

            double retVal = n > 0 && d > 0 ? (i + ((double)n / (double)d)) : i;
            return Math.Round( retVal, precision );
        }

        static private int GCD( int x, int y )
        {
           if ( y == 0 )
               return x;
           
            return GCD(y, x % y);
        }

    }
    #endregion FractionalConversionDataFilter class
}
